@@ -1,3 +1,5 @@
+package AppConfig::File;
+
 #============================================================================
 #
 # AppConfig::File.pm
@@ -10,19 +12,25 @@
 # Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 #
-# $Id: File.pm,v 1.60 2003/04/29 10:43:16 abw Exp $
+# $Id: File.pm,v 1.62 2004/02/04 10:28:28 abw Exp $
 #
 #============================================================================
 
-package AppConfig::File;
+use strict;
 
 require 5.005;
+
 use AppConfig;
 use AppConfig::State;
-use strict;
+use File::HomeDir;
+
 use vars qw( $VERSION );
+BEGIN {
+	$VERSION = '1.64';
+}
+
+
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.60 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -41,16 +49,18 @@ sub new {
     my $state = shift;
     
     my $self = {
-	STATE    => $state,                # AppConfig::State ref
-	DEBUG    => $state->_debug(),      # store local copy of debug 
-	PEDANTIC => $state->_pedantic,     # and pedantic flags
+        STATE    => $state,                # AppConfig::State ref
+        DEBUG    => $state->_debug(),      # store local copy of debug 
+        PEDANTIC => $state->_pedantic,     # and pedantic flags
     };
 
     bless $self, $class;
 
+    # Find the home directory
+    $self->{HOME} = File::HomeDir->my_home;
+
     # call parse(@_) to parse any files specified as further params
-    $self->parse(@_)
-	if @_;
+    $self->parse(@_) if @_;
 
     return $self;
 }
@@ -180,8 +190,21 @@ sub parse {
 	    if (/^([^\s=]+)(?:(?:(?:\s*=\s*)|\s+)(.*))?/) {
 		my ($variable, $value) = ($1, $2);
 
-		# strip any quoting from the variable value
-		$value =~ s/^(['"])(.*)\1$/$2/ if defined $value;
+		if (defined $value) {
+		    # here document
+		    if ($value =~ /^([^\s=]+\s*=)?\s*<<(['"]?)(\S+)\2$/) { # '<<XX' or 'hashkey =<<XX'
+			my $boundary = "$3\n";
+			$value = defined($1) ? $1 : '';
+			while (<$handle>) {
+			    last if $_ eq $boundary;
+			    $value .= $_;
+			};
+			$value =~ s/[\r\n]$//;
+		    } else {
+			# strip any quoting from the variable value
+			$value =~ s/^(['"])(.*)\1$/$2/;
+		    };
+		};
 
 		# strip any leading '+/-' from the variable
 		$variable =~ s/^([\-+]?)//;
@@ -388,20 +411,9 @@ sub _expand {
 		    if ($sys->can_getpwnam()) {
 			$val = ($sys->getpwnam($var))[7];
 		    }
-		}
-		else {
+		} else {
 		    # determine home directory 
-		    unless (defined($val = $self->{ HOME })) {
-			if (exists $ENV{ HOME }) {
-			    $val = $ENV{ HOME };
-			}
-			elsif ($sys->can_getpwuid()) {
-			    $val = ($sys->getpwuid($<))[7];
-			}
-
-			# cache value for next time
-			$self->{ HOME } = $val;
-		    }
+		    $val = $self->{ HOME };
 		}
 
 		# catch-all for undefined $dir
@@ -440,8 +452,11 @@ sub _expand {
 		# expand the variable if defined
 		if (exists $ENV{ $var }) {
 		    $val = $ENV{ $var };
-		}
-		else {
+		} elsif ( $var eq 'HOME' ) {
+		    # In the special case of HOME, if not set
+		    # use the internal version
+		    $val = $self->{ HOME };
+		} else {
 		    # raise a warning if EXPAND_WARN set
 		    if ($expand & AppConfig::EXPAND_WARN) {
 			$state->_error("$var: no such environment variable");
@@ -632,6 +647,30 @@ A reference to the hash is returned when the variable is requested.
 	print "$k => $aliases->{ $k }\n";
     }
 
+A large chunk of text can be defined using Perl's "heredoc" quoting
+style.
+
+   scalar = <<BOUNDARY_STRING
+   line 1
+   line 2: Space/linebreaks within a HERE document are kept.
+   line 3: The last linebreak (\n) is stripped.
+   BOUNDARY_STRING
+
+   hash   key1 = <<'FOO'
+     * Quotes (['"]) around the boundary string are simply ignored.
+     * Whether the variables in HERE document are expanded depends on
+       the EXPAND option of the variable or global setting.
+   FOO
+
+   hash = key2 = <<"_bar_"
+   Text within HERE document are kept as is.
+   # comments are treated as a normal text.
+   The same applies to line continuation. \
+   _bar_
+   
+Note that you cannot use HERE document as a key in a hash or a name 
+of a variable.
+
 The '-' prefix can be used to reset a variable to its default value and
 the '+' prefix can be used to set it to 1
 
@@ -648,10 +687,10 @@ Three different expansion types may be applied:
 
     bin = ~/bin          # expand '~' to home dir if EXPAND_UID
     tmp = ~abw/tmp       # as above, but home dir for user 'abw'
-
+    
     perl = $bin/perl     # expand value of 'bin' variable if EXPAND_VAR
     ripl = $(bin)/ripl   # as above with explicit parens
-
+    
     home = ${HOME}       # expand HOME environment var if EXPAND_ENV
 
 See L<AppConfig::State> for more information on expanding variable values.
@@ -675,7 +714,7 @@ Andy Wardley, E<lt>abw@wardley.orgE<gt>
 
 =head1 REVISION
 
-$Revision: 1.60 $
+$Revision: 1.62 $
 
 =head1 COPYRIGHT
 
